@@ -27,6 +27,7 @@ from IPython.core.magic import (
 from IPython.core.magic_arguments import magic_arguments, argument, parse_argstring
 import sparkmagic.utils.configuration as conf
 from ..utils.resource_check import check_host_and_port, is_ssl_enabled
+from ..utils.arn_util import RoleArnValidator
 
 # The class MUST call this class decorator at creation time
 # https://ipython.readthedocs.io/en/stable/config/custommagics.html
@@ -468,6 +469,9 @@ def _validate_emr_args(args, usage, kernel_name):
             )
         )
 
+    if args.assumable_role_arn:
+        RoleArnValidator.validate(args.assumable_role_arn)
+
     # Only IPython kernel needs language option support
     if kernel_name == IPYTHON_KERNEL:
         if args.language is None:
@@ -488,7 +492,11 @@ def _initiate_emr_connect(args, boto_session, kernel_name):
     # Needs EMR list instance group permissions
     emr_client = boto_session.client("emr")
 
-    cluster = EMRCluster(cluster_id=args.cluster_id)
+    """
+    Even if assumable role arn is None, we can pass it to the EMRCluster constructor and will be handled by
+    _get_boto3_session method call gracefully. So no need to do a explicit None check here.
+    """
+    cluster = EMRCluster(cluster_id=args.cluster_id, role_arn=args.assumable_role_arn)
     _validate_cluster_auth_with_auth_type_provided(
         auth_type=args.auth_type, emr_cluster=cluster
     )
@@ -620,6 +628,12 @@ class SagemakerAnalytics(Magics):
             LANGUAGES_SUPPORTED
         ),
     )
+    @argument(
+        "--assumable-role-arn",
+        type=str,
+        default=None,
+        help="The IAM role to assume when connecting to a cluster in a different account.",
+    )
     def sm_analytics(self, line):
         usage = "Please look at usage of %sm_analytics by executing `%sm_analytics?`."
         user_input = line
@@ -638,7 +652,7 @@ class SagemakerAnalytics(Magics):
                     _validate_emr_args(args=args, usage=usage, kernel_name=kernel_name)
 
                     # Only create boto session if absolutely needed
-                    boto_session = _get_boto3_session()
+                    boto_session = _get_boto3_session(args.assumable_role_arn)
                     _initiate_emr_connect(
                         args=args, boto_session=boto_session, kernel_name=kernel_name
                     )
@@ -672,8 +686,30 @@ def load_ipython_extension(ipython):
     ipython.register_magics(SagemakerAnalytics)
 
 
-def _get_boto3_session():
-    return boto3.session.Session()
+def _get_boto3_session(role_arn):
+    """
+    Allows cross account cluster access
+    """
+    if not role_arn:
+        return boto3.session.Session()
+    else:
+        logger.info(f"Assuming role: {role_arn}")
+        sts_client = boto3.client("sts")
+        try:
+            assume_role_object = sts_client.assume_role(
+                RoleArn=role_arn, RoleSessionName="SageMakerStudioUser"
+            )
+        except botocore.exceptions.ClientError as ce:
+            logger.debug(f"Failed to assume role: ({role_arn}) details. {ce.response}")
+            raise ValueError(
+                f"Unable to assume role(arn: {role_arn}). Ensure permissions are setup correctly"
+                f' Error: {ce.response["Error"]}'
+            ) from None
+        return boto3.Session(
+            aws_access_key_id=assume_role_object["Credentials"]["AccessKeyId"],
+            aws_secret_access_key=assume_role_object["Credentials"]["SecretAccessKey"],
+            aws_session_token=assume_role_object["Credentials"]["SessionToken"],
+        )
 
 
 def _list_instance_groups(emr_client, cluster_id):
